/**
* Title: 		 MyGeneric<T>
* Description:	 Class testing some basic Java 5 "Generic" features
* Copyright:     Copyright (c) 2013-2015
* Company:       Morpheus Quality Technologies
* Creation date: 30-12-2014
* Last modified: 10-01-2015
* @author 		 Stefano Adriani
* @version 		 1.0
*/

package adriani.tutorial.jfive.generic ;

 /**
 * A Generic class is defined using following format:<br/><br/>
 * 
 * <code>class name&lt;T1, T2, ..., Tn&gt; { ...  }</code><br/><br/>
 * 
 * The type parameter section, delimited by angle brackets, specifies the <strong>type parameters</strong> 
 * (also called <em>type variables</em>) T1, T2, ..., and Tn. Types can be anything non-primitive.<br/><br/>
 * 
 * It's not mandatory to use the "T" keyword, but it's useful to respect the standard naming convention.
 * For example "T" is used for Types, "K" for Key, "N" for Number, "E" for a collection Element, "V" for Value.
 */
public class MyGeneric<T> {

	/** 
	* The class field hosting the Generic type parameter. If this class is created as a "raw type", then
	* this field is treated as a Generic <code>Object</code> type.
	*/
    protected 		T 		myType ;
    
    /**
    * Stateless constructor.
    */
    public MyGeneric()			{ }
    
    /**
    * Stateful constructor.
    *  
    * @param myType the value of the type argument (matching the class type parameter) 
    */
    public MyGeneric(T myType)	{ 			this.myType = myType ; 						}
    
    /**
    * Setter method.
    *  
    * @param myType the value of the type argument (matching the class type parameter) 
    */
    public void set(T myType)	{ 			this.myType = myType ; 						}
    
    /** 
    * Getter method. 		
    *  			
    * @return the value of the type argument (matching the class type parameter)
    */
    public T get() 				{ 			return myType ; 							}
    
    /** 
    * Dummy method, used to test some advanced inheritance feature of Generic types. 		
    *  			
    * @return a string containing the Generic type wrapped by this class
    */
    public String format() 		{ 			return ("Wrapped Type = " + myType) ;		}
    
} // End of class