/**
* Title: 		 TypeInference
* Description:	 Simple class helping to test the Type Inference feature 
* Copyright:     Copyright (c) 2014
* Company:       Morpheus Quality Technologies
* Creation date: 30-12-2014
* Last modified: 10-01-2015
* @author 		 Stefano Adriani
* @version 		 1.0
*/

package adriani.tutorial.jfive.generic ;

import java.util.* ;

/**
* Simple class exposing some methods easing testing of the TYPE INFERENCE feature.<br/><br/>
*  
* TYPE INFERENCE basically helps to save typing, since it allows a shorter syntax. However, in  
* some scenarios, ignoring such feature could result in a compilation error. An example is shown within  
* the <code>GenericTester</code> class, where we can't execute the call:<br/><br/>
* 
* <code>typeInference.typedMethod( Collections.emptyList() ) ;</code><br/><br/>
* 
* because we forgot that TYPE INFERENCE does not work in this situation. 
* 
*  @see GenericTester#testTypeInference()
*/
public class TypeInference<T> {

	/** Class field hosting the Generic type parameter (the one defined at class level)						*/
    private 		 		T 		myType ;
    
  	/** Beginning of each logging message produced by this class.											*/
	protected static		String		leadingName 		= "TypeInference - " ;
    
    /**
    * Basic constructor for the Generic instance.
    */
	public TypeInference(T myType) {
		
		this.myType		= myType ;
	}
	
// *********************************************************************************************************
// **********************************			METHODS 		********************************************
// *********************************************************************************************************
	
    /**
    * Simple Generic method, requiring two typed parameters: the 1st is defined at class level (T), while 
    * the 2nd is defined by the method's signature (X).
    * 
    * @param classType a typed parameters matching the type defined at class level
    * @param methodType a typed parameters matching the type defined when invoking the method
    * @return a Generic List, matching the type defined when invoking the method
    */
	public <X> List<X> genericMethod(T classType, X methodType) {
		
		List<X> list = new ArrayList<X>() ;
		list.add(methodType) ;
		return list ;
	}
	
    /**
    * Simple typed method, requiring a NOT Generic argument. This allows us to check if objects defined
    * assuming TYPE INFERENCE are correctly recognized by the compiler 
    * (see the <code>GenericTester</code> class).
    * 
    * @param a string List (i.e. a "Generic type" with not-generic type parameter)  
    */
	public void typedMethod(List<String> list) {
		
		System.out.println(leadingName + "List size: " + list.size() ) ;
	}
	
// *********************************************************************************************************
// **********************************		SETTERS/GETTERS 	********************************************
// *********************************************************************************************************
	
	/** Setter method. 														*/
    public void setValue(T value) 		{ this.myType 	= value ; 			}
    
    /** Getter method. 														*/
    public T getValue() 				{ return this.myType ; 				}
    
} // End of class